﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Microsoft.IdentityModel.Clients.ActiveDirectory;
using System.Threading.Tasks;
using System.Net.Http;
using System.Collections.Specialized;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System.IdentityModel.Tokens;
using System.Net;
using System.IO;
using System.Web.Security;

namespace PBIWebApp
{
     /* NOTE: This sample is to illustrate how to authenticate a Power BI web app. 
     * In a production application, you should provide appropriate exception handling and refactor authentication settings into 
     * a secure configuration. Authentication settings are hard-coded in the sample to make it easier to follow the flow of authentication. */
    public partial class Dashboard : Page
    {
        string baseUri = "https://api.powerbi.com/beta/myorg/";

        public AuthenticationResult authResult { get; set; }

        protected void Page_Load(object sender, EventArgs e)
        {

            //Test for AuthenticationResult
            if (Session["authResult"] != null)
            {
                //Get the authentication result from the session
                authResult = (AuthenticationResult)Session["authResult"];

                //Set user and toek from authentication result
                userLabel.Text = Session["user"].ToString();
                accessTokenTextbox.Text = authResult.AccessToken;
            }

            else
            {
                FormsAuthentication.RedirectToLoginPage();
            }
        }

  
        //Get dashboards
        protected void getDashboardsButton_Click(object sender, EventArgs e)
        {
            string responseContent = string.Empty;

            //Configure datasets request
            System.Net.WebRequest request = System.Net.WebRequest.Create(String.Format("{0}dashboards", baseUri)) as System.Net.HttpWebRequest;
            request.Method = "GET";
            request.ContentLength = 0;
            request.Headers.Add("Authorization", String.Format("Bearer {0}", authResult.AccessToken));

            //Get datasets response from request.GetResponse()
            using (var response = request.GetResponse() as System.Net.HttpWebResponse)
            {
                //Get reader from response stream
                using (var reader = new System.IO.StreamReader(response.GetResponseStream()))
                {
                    responseContent = reader.ReadToEnd();

                    //Deserialize JSON string
                    PBIDashboards PBIDashboards = JsonConvert.DeserializeObject<PBIDashboards>(responseContent);

                    tb_dashboardsResult.Text = string.Empty;
                    //Get each Dataset from 
                    foreach (PBIDashboard db in PBIDashboards.value)
                    {
                        tb_dashboardsResult.Text += String.Format("{0}\t{1}\n", db.id, db.displayName);
                    }
                }
            }
        }

        //Get tiles for the selected dashboard
        protected void getTilesButton_Click(object sender, EventArgs e)
        {
            string responseContent = string.Empty;
            string dashboardId = string.Empty;

            if (string.Empty == inDashboardID.Text)
            {
                tb_tilesResult.Text = "Please enter a dashboard id above"; return;
            }
            dashboardId = inDashboardID.Text;

            //Configure datasets request
            System.Net.WebRequest request = System.Net.WebRequest.Create(String.Format("{0}Dashboards/{1}/Tiles", 
                    baseUri, dashboardId)) as System.Net.HttpWebRequest;
            request.Method = "GET";
            request.ContentLength = 0;
            request.Headers.Add("Authorization", String.Format("Bearer {0}", authResult.AccessToken));

            using (var response = request.GetResponse() as System.Net.HttpWebResponse)
            {
                //Get reader from response stream
                using (var reader = new System.IO.StreamReader(response.GetResponseStream()))
                {
                    responseContent = reader.ReadToEnd();
                    //Deserialize JSON string
                    PBITiles PBITiles = JsonConvert.DeserializeObject<PBITiles>(responseContent);
                    tb_tilesResult.Text = string.Empty;
                    foreach (PBITile tile in PBITiles.value)
                    {
                        tb_tilesResult.Text += String.Format("{0}\t{1}\t{2}\n", tile.id, tile.title, tile.embedUrl);
                    }
                }
            }
        }

    }
    
    //Power BI Datasets
    public class PBIDatasets
    {
        public Dataset[] Datasets { get; set; }
    }

    public class PBIGroups
    {
        public PBIGroup[] value { get; set; }
    }

    public class Dataset
    {
        public string Id { get; set; }
        public string Name { get; set; }
    }

    public class PBIGroup
    {
        public string id { get; set; }
        public string name {get; set;}
    }

    public class PBIDashboards
    {
        public PBIDashboard[] value { get; set; }
    }
    public class PBIDashboard
    {
        public string id { get; set; }
        public string displayName { get; set; }
    }

    public class PBITiles
    {
        public PBITile[] value { get; set; }
    }
    public class PBITile
    {
        public string id { get; set; }
        public string title { get; set; }
        public string embedUrl { get; set; }
    }

}